# -*- encoding: utf-8 -*-
# @Time     :  2025/04/07 01:42:07
# @Author   :  Yizheng Wang
# @E-mail   :  wyz020@126.com
# @Function :  使用SBSM算法的示例脚本

from sbsm import SBSM  # 从sbsm库中导入SBSM类


def read_fasta(file_path):
    """
    从FASTA格式文件中读取核酸序列。
    """
    with open(file_path, 'r') as f:
        lines = f.readlines()
        even_lines = [line.strip() for line in lines[1::2]]
        return even_lines


def read_label(file_path):
    """
    从txt文本中读取标签数据。
    """
    with open(file_path, 'r') as file:
        content_list = [int(line.strip()) for line in file.readlines()]
    return content_list


def evaluate_metrics(true_labels, predictions):
    """
    评估二分类问题的关键指标。
    """
    TP = FP = TN = FN = 0
    for pred, true in zip(predictions, true_labels):
        if pred == 1 and true == 1:
            TP += 1
        elif pred == 1 and true == 0:
            FP += 1
        elif pred == 0 and true == 0:
            TN += 1
        elif pred == 0 and true == 1:
            FN += 1
            
    TPR = TP / (TP + FN) if TP + FN != 0 else 0
    TNR = TN / (TN + FP) if TN + FP != 0 else 0
    ACC = (TP + TN) / (TP + TN + FP + FN) if (TP + TN + FP + FN) != 0 else 0
    MCC_numerator = TP * TN - FP * FN
    MCC_denominator = ((TP + FP) * (TP + FN) * (TN + FP) * (TN + FN))**0.5
    MCC = MCC_numerator / MCC_denominator if MCC_denominator != 0 else 0
    
    return TPR, TNR, ACC, MCC


if __name__ == "__main__":

    # 解决 Windows 下多进程启动问题，避免子进程递归运行
    from multiprocessing import freeze_support
    freeze_support()


    #-------- Step1: 导入所需数据--------#
    train_sequences = read_fasta("train_samples.fasta")
    test_sequences = read_fasta("test_samples.fasta")
    train_labels = read_label("train_labels.txt")
    test_labels = read_label("test_labels.txt")


    #-------- Step2: 定义SBSM模型--------#
    sbsm = SBSM(
        match_score=2,  # 碱基匹配得分
        mismatch_score=-1,  # 碱基不匹配得分
        gap_score=-2,  # 插入/删除惩罚得分
        k_neighbors=15,  # K近邻的数量
        lambda_=0.9,  # 衰减因子
        nu1=0.001,  # 正则项1的权重系数
        nu2=0.001,  # 正则项2的权重系数
        process_num=30  # 线程数
    )


    #-------- Step3: 计算核矩阵--------#
    # 使用SBSM模型计算训练集和测试集的核矩阵，核矩阵用于表示样本之间的相似性
    train_kernel, test_kernel = sbsm.compute_kernels(
        X_train=train_sequences,
        X_test=test_sequences,
        y_train=train_labels
    )


    #-------- Step4: 模型训练--------#
    # 使用训练核矩阵和标签拟合（训练）SBSM模型
    sbsm.fit(
        train_kernel, 
        train_labels, 
        alpha=1.0,  # 权重缩放因子
        C=1.0  # 惩罚系数
    )


    #-------- Step5: 模型预测--------#
    predictions = sbsm.predict(test_kernel)  # 使用测试核矩阵进行预测，返回每个测试样本的预测标签
    probabilities = sbsm.predict_proba(test_kernel)  # 使用测试核矩阵进行预测，返回每个测试样本的预测概率


    # -------- Step6: 结果评估 --------#
    TNR, TPR, ACC, MCC = evaluate_metrics(test_labels, predictions)  # 计算评估指标
    print(f"TPR:{TPR}, TNR:{TNR}, ACC:{ACC}, MCC:{MCC}")  # 打印评估指标
