# -*- encoding: utf-8 -*-
# @Time     :  2025/04/07 01:42:07
# @Author   :  Yizheng Wang
# @E-mail   :  wyz020@126.com
# @Function :  使用SBSM算法的示例脚本（含交叉验证训练与评估）

from sbsm import SBSM  # 从sbsm库中导入SBSM类
import numpy as np


def read_fasta(file_path):
    """
    从FASTA格式文件中读取核酸序列。
    """
    with open(file_path, 'r') as f:
        lines = f.readlines()
        even_lines = [line.strip() for line in lines[1::2]]
        return even_lines


def read_label(file_path):
    """
    从txt文本中读取标签数据。
    """
    with open(file_path, 'r') as file:
        content_list = [int(line.strip()) for line in file.readlines()]
    return content_list


def evaluate_metrics(true_labels, predictions):
    """
    评估二分类问题的关键指标。
    """
    TP = FP = TN = FN = 0
    for pred, true in zip(predictions, true_labels):
        if pred == 1 and true == 1:
            TP += 1
        elif pred == 1 and true == 0:
            FP += 1
        elif pred == 0 and true == 0:
            TN += 1
        elif pred == 0 and true == 1:
            FN += 1

    TPR = TP / (TP + FN) if TP + FN != 0 else 0
    TNR = TN / (TN + FP) if TN + FP != 0 else 0
    ACC = (TP + TN) / (TP + TN + FP + FN) if (TP + TN + FP + FN) != 0 else 0
    MCC_numerator = TP * TN - FP * FN
    MCC_denominator = ((TP + FP) * (TP + FN) * (TN + FP) * (TN + FN))**0.5
    MCC = MCC_numerator / MCC_denominator if MCC_denominator != 0 else 0

    return TPR, TNR, ACC, MCC


if __name__ == "__main__":

    # 解决 Windows 下多进程启动问题，避免子进程递归运行
    from multiprocessing import freeze_support
    freeze_support()


    #-------- Step1: 导入所需数据--------#
    sequences = read_fasta("train_samples.fasta")
    labels = read_label("train_labels.txt")


    #-------- Step2: 定义SBSM模型--------#
    sbsm = SBSM(
        match_score=2,
        mismatch_score=-1,
        gap_score=-2,
        k_neighbors=15,
        lambda_=0.9,
        nu1=0.001,
        nu2=0.001,
        process_num=30
    )


    #-------- Step3: 计算交叉验证核矩阵--------#
    cv_kernel = sbsm.compute_kernels_cv(
        X_train=sequences,
        y_train=labels,
        k_fold=5,
        shuffle=True,
        random_state=0,
    )


    #-------- Step4: 交叉验证--------#
    tpr_list, tnr_list, acc_list, mcc_list = [], [], [], []

    for i, (K_train, K_test, y_train, y_test) in enumerate(cv_kernel):

        sbsm.fit(K_train, y_train, alpha=0.5, C=64.0)
        predictions = sbsm.predict(K_test)
        TPR, TNR, ACC, MCC = evaluate_metrics(y_test, predictions)
        print(f"Fold{i}: TPR:{TPR:.4f}, TNR:{TNR:.4f}, ACC:{ACC:.4f}, MCC:{MCC:.4f}")
        tpr_list.append(TPR)
        tnr_list.append(TNR)
        acc_list.append(ACC)
        mcc_list.append(MCC)


    #-------- Step5: 输出平均评估结果--------#
    print(f" Mean: TPR:{np.mean(tpr_list):.4f}, TNR:{np.mean(tnr_list):.4f}, ACC:{np.mean(acc_list):.4f}, MCC:{np.mean(mcc_list):.4f}")
